<?php

namespace Controller;

use Service\DeleteCascadeService;
use Service\PeriodMathExpression\Period;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Utilities;
use Service\Traits\TResponse;

class Periods extends Crud
{

    use TResponse;

    protected $dataSourceName = 'periods';
    /** @var  \Service\PeriodsService */
    protected $periodsService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, $periodsService)
    {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->periodsService = $periodsService;
        $this->orderFields = [];
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {

        $params = $request->getParameters();
        $params["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
        $params["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
        $filter = Utilities::parseParamNextToZmartFilterCriteria($this->dataSourceName, $params);
        $periods = $this->findAllPeriods($params);
        $periods = Utilities::parseDataSourceZeedhiNext($periods);
        $periods['data'] = $this->fillEmptyAttributes($periods['data']);

        $periods = $this->validateParameters($periods, $params);

        $this->sendResponse($periods, 200);
    }

    public function getDateByDateExpression(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow()->get('datesExpression');
        $row = $this->periodsService->getDateByDateExpression($row);

        $this->sendResponse($row, 200);
    }

    /**
     * @return array
     */
    public function findAllPeriods($params)
    {
        $filterCriteria = Utilities::parseParamNextToZmartFilterCriteria($this->dataSourceName, $params);
        $filterCriteria->setPage(1);
        $filterCriteria->setPageSize(9999);
        $basePeriods = $this->periodsService->getPeriodsFromBase($filterCriteria);
        $orgPeriods = $this->periodsService->getPeriodsFromOrg($filterCriteria);
        $periods = array_merge($basePeriods, $orgPeriods);
        foreach ($periods as $key => &$period) {
            $startDateExpression = !empty($period['START_DATE_EXPRESSION']) ? $period['START_DATE_EXPRESSION'] : null;
            $endDateExpression = !empty($period['END_DATE_EXPRESSION']) ? $period['END_DATE_EXPRESSION'] : null;
            if ($startDateExpression || $endDateExpression) {
                try {

                    $period['interval'] = [
                        'start' => Period::getDate($startDateExpression),
                        'end' => Period::getDate($endDateExpression),
                        '_id' => $period['_id'],
                        'customized' => true
                    ];
                } catch (\Exception $e) {
                    $period['Erro'] = $e->getMessage();
                }
            } else {
                unset($period['interval']);
            }
        }
        return $periods;
    }

    private function fillEmptyAttributes($periods)
    {
        foreach ($periods as $key => $period) {
            $periods[$key]['name'] = array_key_exists('name', $period) ? $period['name'] : '';
            $periods[$key]['labelOnView'] = array_key_exists('labelOnView', $period) ? $period['labelOnView'] : '';
        }

        return $periods;
    }

    private function validateParameters($periods, $params)
    {
        if (array_key_exists("order", $params)) {
            $periodData = $periods['data'];
            $order = $params['order'];
            
            foreach($order as &$orderField) {
                $orderField = explode(".", $orderField);
            }
            
            $this->orderFields = $order;
            usort($periodData, array($this, 'comparator'));
            $periods['data'] = $periodData;
        }
        
        $limit = -1;
        if (array_key_exists("limit", $params)) {
            $limit = intval($params['limit']);
            $periods['pagination']['limit'] = $limit;
            $periodData = sizeof($periods['data']) > 0 && $limit > 0 ? array_chunk($periods['data'], $limit) : array();
        }

        $page = 1;
        if (array_key_exists("page", $params)) {
            $page = intval($params['page']);
            $periods['pagination']['page'] = $page;
            if ($limit != -1 && sizeof($periodData) >= $page) $periods['data'] = $periodData[$page - 1];
        }
        
        return $periods;
    }
    
    private function comparator($object1, $object2)
    {
        $orderList = $this->orderFields;
        foreach($orderList as $order) {
            $orderResult = ($order[1] == 'asc') ? $this->comparatorAsc($object1, $object2, $order[0]) : $this->comparatorDesc($object1, $object2, $order[0]);
            if ($orderResult)
                return $orderResult;
        }
        return 0;
    }

    private function comparatorAsc($object1, $object2, $field)
    {
        return strcmp(strtolower($object1[$field]), strtolower($object2[$field]));
    }

    private function comparatorDesc($object1, $object2, $field)
    {
        return strcmp(strtolower($object2[$field]), strtolower($object1[$field]));
    }
}
